@@ -10,7 +10,7 @@
     :license: BSD, see LICENSE for more details.
 """
 
-nodeenv_version = '0.8.2'
+nodeenv_version = '0.9.0'
 
 import sys
 import os
@@ -376,6 +376,68 @@ def get_node_src_url_postfix(opt):
 # Virtual environment functions
 
 
+def copy_node_from_prebuilt(env_dir, src_dir):
+    """
+    Copy prebuilt binaries into environment
+    """
+    logger.info('.', extra=dict(continued=True))
+    callit(['cp', '-r', src_dir + '/node-v*/*', env_dir], True, env_dir)
+    logger.info('.', extra=dict(continued=True))
+
+
+def build_node_from_src(env_dir, src_dir, node_src_dir, opt):
+    env = {}
+    make_param_names = ['load-average', 'jobs']
+    make_param_values = map(
+        lambda x: getattr(opt, x.replace('-', '_')),
+        make_param_names)
+    make_opts = [
+        '--{0}={1}'.format(name, value)
+        if len(value) > 0 else '--{0}'.format(name)
+        for name, value in zip(make_param_names, make_param_values)
+        if value is not None
+    ]
+
+    if sys.version_info.major > 2:
+        # Currently, the node.js build scripts are using python2.*,
+        # therefore we need to temporarily point python exec to the
+        # python 2.* version in this case.
+        try:
+            _, which_python2_output = callit(['which', 'python2'])
+            python2_path = which_python2_output[0].decode('utf-8')
+        except (OSError, IndexError):
+            raise OSError(
+                'Python >=3.0 virtualenv detected, but no python2 '
+                'command (required for building node.js) was found'
+            )
+        logger.debug(' * Temporarily pointing python to %s', python2_path)
+        node_tmpbin_dir = join(src_dir, 'tmpbin')
+        node_tmpbin_link = join(node_tmpbin_dir, 'python')
+        mkdir(node_tmpbin_dir)
+        if not os.path.exists(node_tmpbin_link):
+            callit(['ln', '-s', python2_path, node_tmpbin_link])
+        env['PATH'] = '{}:{}'.format(node_tmpbin_dir,
+                                     os.environ.get('PATH', ''))
+
+    conf_cmd = []
+    conf_cmd.append('./configure')
+    conf_cmd.append('--prefix=%s' % pipes.quote(env_dir))
+    if opt.without_ssl:
+        conf_cmd.append('--without-ssl')
+    if opt.debug:
+        conf_cmd.append('--debug')
+    if opt.profile:
+        conf_cmd.append('--profile')
+
+    make_cmd = opt.make_path
+
+    callit(conf_cmd, opt.verbose, True, node_src_dir, env)
+    logger.info('.', extra=dict(continued=True))
+    callit([make_cmd] + make_opts, opt.verbose, True, node_src_dir, env)
+    logger.info('.', extra=dict(continued=True))
+    callit([make_cmd + ' install'], opt.verbose, True, node_src_dir, env)
+
+
 def install_node(env_dir, src_dir, opt):
     """
     Download source code for node.js, unpack it
@@ -395,60 +457,9 @@ def install_node(env_dir, src_dir, opt):
     logger.info('.', extra=dict(continued=True))
 
     if opt.prebuilt:
-        logger.info('.', extra=dict(continued=True))
-        callit(['cp', '-r', src_dir + '/node-v*/*', env_dir], True, env_dir)
-        logger.info('.', extra=dict(continued=True))
+        copy_node_from_prebuilt(env_dir, src_dir)
     else:
-        env = {}
-        make_param_names = ['load-average', 'jobs']
-        make_param_values = map(
-            lambda x: getattr(opt, x.replace('-', '_')),
-            make_param_names)
-        make_opts = [
-            '--{0}={1}'.format(name, value)
-            if len(value) > 0 else '--{0}'.format(name)
-            for name, value in zip(make_param_names, make_param_values)
-            if value is not None
-        ]
-
-        if sys.version_info.major > 2:
-            # Currently, the node.js build scripts are using python2.*,
-            # therefore we need to temporarily point python exec to the
-            # python 2.* version in this case.
-            try:
-                _, which_python2_output = callit(['which', 'python2'])
-                python2_path = which_python2_output[0].decode('utf-8')
-            except (OSError, IndexError):
-                raise OSError(
-                    'Python >=3.0 virtualenv detected, but no python2 '
-                    'command (required for building node.js) was found'
-                )
-            logger.debug(' * Temporarily pointing python to %s', python2_path)
-            node_tmpbin_dir = join(src_dir, 'tmpbin')
-            node_tmpbin_link = join(node_tmpbin_dir, 'python')
-            mkdir(node_tmpbin_dir)
-            if not os.path.exists(node_tmpbin_link):
-                callit(['ln', '-s', python2_path, node_tmpbin_link])
-            env['PATH'] = '{}:{}'.format(node_tmpbin_dir,
-                                         os.environ.get('PATH', ''))
-
-        conf_cmd = []
-        conf_cmd.append('./configure')
-        conf_cmd.append('--prefix=%s' % pipes.quote(env_dir))
-        if opt.without_ssl:
-            conf_cmd.append('--without-ssl')
-        if opt.debug:
-            conf_cmd.append('--debug')
-        if opt.profile:
-            conf_cmd.append('--profile')
-
-        make_cmd = opt.make_path
-
-        callit(conf_cmd, opt.verbose, True, node_src_dir, env)
-        logger.info('.', extra=dict(continued=True))
-        callit([make_cmd] + make_opts, opt.verbose, True, node_src_dir, env)
-        logger.info('.', extra=dict(continued=True))
-        callit([make_cmd + ' install'], opt.verbose, True, node_src_dir, env)
+        build_node_from_src(env_dir, src_dir, node_src_dir, opt)
 
     logger.info(' done.')
 
@@ -502,18 +513,20 @@ def install_activate(env_dir, opt):
     """
     Install virtual environment activation script
     """
-    files = {'activate': ACTIVATE_SH}
+    files = {'activate': ACTIVATE_SH, 'shim': SHIM}
     bin_dir = join(env_dir, 'bin')
     mod_dir = join('lib', 'node_modules')
     prompt = opt.prompt or '(%s)' % os.path.basename(os.path.abspath(env_dir))
     mode_0755 = (stat.S_IRWXU | stat.S_IXGRP |
                  stat.S_IRGRP | stat.S_IROTH | stat.S_IXOTH)
+    shim_node = "`which node`" if opt.node == "system" else bin_dir
 
     for name, content in files.items():
         file_path = join(bin_dir, name)
         content = content.replace('__NODE_VIRTUAL_PROMPT__', prompt)
         content = content.replace('__NODE_VIRTUAL_ENV__',
                                   os.path.abspath(env_dir))
+        content = content.replace('__SHIM_NODE__', shim_node)
         content = content.replace('__BIN_NAME__', os.path.basename(bin_dir))
         content = content.replace('__MOD_NAME__', mod_dir)
         writefile(file_path, content, append=opt.python_virtualenv)
@@ -629,6 +642,13 @@ ENABLE_PROMPT = """
 unset NODE_VIRTUAL_ENV_DISABLE_PROMPT
 """
 
+SHIM = """
+#!/bin/sh
+export NODE_PATH=__NODE_VIRTUAL_ENV__/lib/node_modules
+export NPM_CONFIG_PREFIX=__NODE_VIRTUAL_ENV__
+exec __SHIM_NODE__/node $*
+"""
+
 ACTIVATE_SH = """
 
 # This file must be used with "source bin/activate" *from bash*
